#ifndef HARDDRIVE_H
#define HARDDRIVE_H
#include <qobject.h>
#include <qstring.h>
#include <qptrlist.h> 

/*
        This type be pre-defined is for the measure of
        volumes, or extents, in `mebibyte'. So, any variables
        that is declared with this type should be refered
        to in the meaning as above.
*/
typedef unsigned long int extentsize_t;

static const QString TOKEN_DRIVE        = "DRIVE";
static const QString TOKEN_EXTENT       = "PART";

/*******************************************************************      
* 
*               class Partition
*               this class represents both the physical Disk Partition and the
*               Physical Volume in LVM
*
*******************************************************************/
class Partition : public QObject
{
    Q_OBJECT

public:
    Partition( const QString &aName, const QString &aStrAttrib, const bool pv = false,  QObject * parent = 0, const char * name = 0 );
    virtual ~Partition();
    
    QString path() const; 
    QString name() const;
    int proportion() const;
    QString mountPoint() const;
    QString getSize() const;
    QString getSystem() const;
    void setSystem( const QString aId, const QString aSystem );
    QString fstype() const;
    
    //whether this Partition is a PV
    bool isPv() const;
    //whether this Partition is free, viz., is not occupied by any VG
    bool isAllocatable() const;
    //returns a QString representing the actual LVs for which this PV    
    //provides extents. The string is composed by LV paths seperated by colons(:)    
    //, for example, /dev/vg01/lvol1:/dev/vg01/lvol2
    QString lvString() const;
    void setLvs( const QString & aLvs );
    QString getVgPath() const;
    void setVgPath( const QString & aVgPath );        
    
private:
    QString myName;
    QString strAttrib;     
    int size; //size of this Partition in MBs
    QString id; //the partition's system id
    QString system; //description of the partition's system id, e.g., Linux, HPFS/NTFS.  
    QString mount; //the mount point the this partition (if it is mounted)
    QString fs; //fs type on this partition

    bool bPv; //whether this Partition is a PV
    QString uuid; //uuid of this PV
    QString lvs; //the LVs' names in a ':'-separated string
    QString vgPath; //path of the VG to which this PV belongs
   
    //get the basic attributes of partition
    void getBasics();
    //get to know which LVs this partition harbours
    void getLvs();
    
};

/*******************************************************************
*                                                            
*               class Disk
*              This class represents the Disk Drives 
*
*******************************************************************/
class Disk : public QObject
{
    Q_OBJECT
    
public:    
    Disk( const QString &aName, QObject * parent = 0, const char * name = 0 );
    virtual ~Disk();    
        
    //return a pointer to the specified partition(by sequence num) in partitionList
    Partition * getPartition( uint num );
    //return a pointer to the Partition by its name
    Partition * getPartition( const QString & partitionName );        
    //return the number of partitions in partitionList
    uint partitionCount() const; 
    
    QString path() const;
    QString name() const;
    QString lvString() const;
    
    QString getSize();
    
private:    
    extentsize_t mySize;
    QString myName;

    void setSize( extentsize_t size );
    //find out the partitions in the disk and initialize them,
    //also put the pointer to the partitions into partitionList
    void findPartition();
    //create a partition object and put the pointer to it into partitionList
    void addPartition( const QString & partitionName, const QString & attribs );
    //a list of pointers to the partitions
    QPtrList<Partition> partitionList;
    
    //Hard uses findPartition()
    friend class Hard;
};

/*******************************************************************
*                                                            
*               class Hard
*          This class represents the Harddrive system which contains disks and partitions
*
*******************************************************************/
class Hard : public QObject
{
    Q_OBJECT
    
public:
    Hard( QObject * parent = 0, const char * name = 0 );
    virtual ~Hard();
    
    //return a pointer to the specified Disk(by sequence num) in diskList
    Disk * getDisk( uint num );
    //return a pointer to the Disk by its name
    Disk * getDisk( const QString & diskName );    
    //return the number of Disks in diskList
    uint diskCount() const; 
    //From the PV's path provided in the argument, return
    //the pointer to the Partition object
    Partition * getPv( const QString & pvPath );
    //One VG was created from some of the Partitions(isPv()==true),
    //those Partitions's vgPathes should be set as the newly created VG's path.
    void affiliateVgPv( const QString & aVgPath, const QStringList & listPv );
    //One VG has been removed, the Partitions(isPv()==true) whose 
    //vgPathes refer to that VG should have their vgPathes zeroed
    void deaffiliateVgPv( const QStringList & listPv );
    //One VG has been renamed, the Partitions(isPv()==true) whose 
    //vgPathes refer to that VG should have their vgPathes and lvs' updated    
    void reaffiliateVgPv( const QString & aVgPath, const QStringList & listPv );
    //One LV was created on some of the Partitions(isPv()==true),
    //the LV's name should be appended to those Partitions's lvs' 
    void affiliateLvPv( const QString & aLvPath, const QStringList & listPv );
    //One LV has been removed, the Partitions(isPv()==true) whose
    //lvs' contain the LV's name should have this name removed from lvs    
    void deaffiliateLvPv( const QString & aLvPath, const QStringList & listPv );
    //One LV has been renamed, the Partitions(isPv()==true) whose
    //lvs' contain the LV's name should have this name changed accordingly 
    void reaffiliateLvPv( const QString & aNewLvPath, const QString & aOldLvPath, const QStringList & listPv );
    
private:
    //find out the disks in the harddrive system and initialize them,
    //also put the pointers to the Disks into diskList
    void findDisk();    
    //create a Disk object and put the pointer to it into diskList
    void addDisk( const QString & diskName, extentsize_t size );    
    
    //a list of pointers to the Disks
    QPtrList<Disk> diskList;
};
#endif
