#ifndef LVMDRIVE_H
#define LVMDRIVE_H
#include <qobject.h>
#include <qstring.h>
#include <qstringlist.h>

/*******************************************************************
*                                                            
*               class Lv
*    This class represents the LV
*
*******************************************************************/
class Lv : public QObject	
{    
    Q_OBJECT
    
public:
    Lv( const QString & aName, const QString & aVgName, const int & aLeSize, const int & aFreeSpace, const bool original = false, const bool snapshot = false, QObject * parent = 0, const char * name = 0 );
    virtual ~Lv();
    
    enum Attribute { Access, Status, InternalNum, Size, AllocLe, CurLe, AllocPolicy, ReadAheadSectors, MajorDevNum, MinorDevNum, LeSize, FreeSpace, FixSize, AllocSize };
    
    bool isActive() const;    
    //whether this LV is an original LV for some snapshots
    bool isOriginal() const;
    //whether this LV is snapshot of another LV
    bool isSnapshot() const;
    int proportion() const; 
    QString path() const;
    QString name() const;
    QString getVgPath() const;
    QStringList getSnapshot() const;
    QString getOriginal() const;
    //add snapshotName to listSnapshot
    void addSnapshot( const QString & snapshotName );
    //remove snapshotName from listSnapshot
    void removeSnapshot( const QString & snapshotName );
    //replace the oldName in listSnapshot by newName
    void renameSnapshot( const QString & newName, const QString & oldName );
    
    //returns a QString representing the actual PVs on which the LV    
    //sits.The string is composed by PV paths seperated by colons(:)    
    //,for example, /dev/hda3:/dev/hda4 or /dev/hda3
    QString pvString() const;
    int getAttribute( enum Attribute attrib ) const;
    QString fstype() const;
    QString mountPoint() const;
    
    void setStatus( int newStatus );  
    void setSize( int newSize );   
    void setLe( int aCurLe, int aAllocLe );
    void setOriginal( const QString & originalLvName );
    
    /*actual command*/
    //the return number indicates the retcode of the actual command,    
    //0 is success, otherwise is error
    int createSnapshot( const QStringList & args );    
    int resize( const QStringList & args );    
    int remove();
    int rename( const QString &aNewName );
    
signals:    
    void newLv( const QString &, bool, bool );
    void removeLv( const QString & );    
    void renameLv( const QString &, const QString & );
    void resizeLv( const QString &, const QStringList & );
    
    void output( const QString & );
    
private:
    QString myName; //LV's name
    const QString & vgName; //name of the VG that this LV belongs to
    int access;  //whether this LV is readonly or read/write
    int status;  //whether this LV is available or not
    int internalNum;  //internal number of this LV
    bool open; //whether this LV is mounted or not                    
    int size;  //size of this LV in Mbytes    
    int allocLe;//allocated number of LEs in this LV   
    int curLe; //current LEs associated to this LV
    int allocPolicy; //allocation policy of this LV
                          //(2 for 'contiguous', 0 for 'next free')
    int readAheadSectors; //read ahead sectors of this LV
    int majorDevNum; //major device number of this LV
    int minorDevNum; //minor device number of this LV
    const int & leSize; //LE size of this LV in Mbytes, equal to peSize in Vg
    const int & freeSpace;//free space in the VG to which this LV belongs
    int fixSize;//the size of the part that can't be removed
    int allocSize;//the size of the part that can be removed
    
    /*int myProportion;  // maps to 'myStretch' in class Label*/
    QString pvs;//the PVs' names in a ':'-separated string
    QString outputString;//the output the actual commands
    QStringList listSnapshot;// store the names of the snapshots of this original LV
    QString original;// store the name of the original LV of this snapshot
    QString fs;
    
    bool bOriginal;
    bool bSnapshot;
    //get the basic attributes of LV
    void getBasics();
    //get to know which PVs this LV sits on
    void getPvs();   
    
};

/*******************************************************************
*                                                            
*               class Vg
*          This class represents the VG
*
*******************************************************************/
class Vg : public QObject
{

    Q_OBJECT
public:    
    Vg( const QString &aName, QObject * parent = 0, const char * name = 0  );
    virtual ~Vg();
    
    enum Attribute { Access, Status, InternalNum, MaxLv, CurLv, OpenLv, MaxLvSize, MaxPv, CurPv, ActPv, Size, PeSize, TotalPe, AllocPe, FreePe, FreeSpace };  
    
    //return a pointer to the specified Lv(by sequence num) in lvList
    Lv * getLv( uint num );
    //return a pointer to the Lv by its name
    Lv * getLv( const QString & lvName );    
    //return the number of Lvs in lvList
    uint lvCount() const;
    //whether the LV with lvName is in this VG
    bool hasLv( const QString & lvName );
    
    bool isActive(); 
    bool isResizable() const; 
    QString name() const;
    QString path() const;
    QString uuidString() const;
    //returns a QString representing the actual PVs which the VG    
    //contains.The string is composed by PV paths seperated by colons(:)    
    //,for example, /dev/hda3:/dev/hda4 or /dev/hda3
    QString pvString() const;
    //return the LVs' names in a ':'-separated string, just like pvString()
    QString lvString();
    //return the output of the actual commands
    //QString output() const;
    int getAttribute( enum Attribute attrib ) const;
    
    /*actual command*/
    //the return number indicates the retcode of the actual command    
    //0 is success, >0 is error
    //make this VG active(available)
    int activate();
    //make this VG not active(unavailable)
    int deactivate();
    //make this VG not resizable
    int freeze();
    //make this VG resizable
    int defreeze();            
    int extend( const QStringList & listPv );
    int reduce( const QStringList & listPv );
    int remove();
    int rename( const QString & aNewName );
    int createLv( const QStringList & args );
    
public slots: 	
    void slotNewLv( const QString & lvName, bool bOriginal, bool bSnapshot );
    void slotRemoveLv( const QString & lvName );    
    void slotRenameLv( const QString & newLvName, const QString & oldLvName );
    void slotResizeLv( const QString & lvName, const QStringList & oldListPv );
    
signals:   
    void removed( const QString & );
    void renamed( const QString &, const QStringList & );
    void extended( const QString &, const QStringList & );
    void reduced( const QStringList & );
    
    void oneLvAdded( const QString &, const QStringList & );
    void oneLvRemoved( const QString &, const QStringList & );
    void oneLvRenamed( const QString &, const QString &, const QStringList & );
    void oneLvResized( const QString &, const QStringList &, const QStringList & );
    void output( const QString & );
    
private:    
    QString myName; //VG's name
    QString uuid; //uuid of this VG
    int access;  //whether this VG is readonly or read/write
    int status;  //whether this VG is available(active) or not, resizable or not
    int internalNum;  //internal number of this VG            
    int maxLv; //maximum number of LVs in this VG
    int curLv; //current number of LVs in this VG, only VGs having 0 curLv can be removed
    int openLv;//open count of all LVs in this VG
    int maxLvSize; //maximum LV size in Gbytes
    int maxPv; //maximum number of PVs in this VG
    int curPv; //current number of PVs in this VG
    int actPv; //actual number of PVs in this VG
    int size;  //size of this VG in Mbytes
    int peSize;//PE size of this VG in Mbytes
    int totalPe;//total number of PEs in this VG
    int allocPe;//allocated number of PEs in this VG
    int freePe;//free number of PEs in this VG 
    int freeSpace;//free space in this VG

    QString pvs;//the PVs' names in a ':'-separated string
    QString outputString;//the output the actual commands
    
    //a list of pointers to the Lvs
    QPtrList<Lv> lvList;
    //find out the LVs in this VG and initialize them,
    //also put the pointers to the lvs into lvList
    void findLv();    
    //create a Lv object and put the pointer to it into lvList
    void addLv( const QString & lvName, const bool bOriginal = false, const bool bSnapshot = false);
    //remove the Lv at the position of num in lvList
    bool removeLv( uint num );
    //remove the Lv by its name
    bool removeLv( const QString & lvName );
    
    //get the basic attributes of VG
    void getBasics();
    //get to know which PVs are in this VG
    void getPvs();
    
    //Lvm uses findLv()
    friend class Lvm;
    //Lv references  myName, peSize, freeSpace
    friend class Lv;
};

/*******************************************************************
*                                                            
*               class Lvm
*          This class represents the LVM system which contains VGs and LVs
*
*******************************************************************/
class Lvm : public QObject	
{
    Q_OBJECT
    
public:
    Lvm( QObject * parent = 0, const char * name = 0 );
    virtual ~Lvm();
        
    //return a pointer to the specified Vg(by sequence num) in vgList
    Vg * getVg( uint num );
    //return a pointer to the Vg by its name
    Vg * getVg( const QString & vgName );
    //return the number of Vgs in vgList
    uint vgCount() const; 
    //whether the VG with vgName is in LVM
    bool hasVg( const QString & vgName );
    
    QStringList errorVgName() const;
    
    /*actual command*/
    int createVg( const QStringList & args );
    
public slots: 	    
    void slotRemoveVg( const QString & vgName );
   
signals:
    void output( const QString & );
    void oneVgAdded( const QString &, const QStringList & );
    void oneVgRemoved( const QStringList & );
    void oneVgRenamed( const QString &, const QStringList & );
    void oneVgExtended( const QString &, const QStringList & );
    void oneVgReduced( const QStringList & );
    
    void oneLvAdded( const QString &, const QStringList & );
    void oneLvRemoved( const QString &, const QStringList & );
    void oneLvRenamed( const QString &, const QString &, const QStringList & );
    void oneLvResized( const QString &, const QStringList &, const QStringList & );
    
private:
    QString outputString;//the output of the actual commands
    //find out the VGs in the LVM system and initialize them,
    //also put the pointers to the Vgs into vgList
    void findVg();
    //create a Vg object and put the pointer to it into vgList
    void addVg( const QString & vgName );
    //remove the Vg at the position of num in vgList
    bool removeVg( uint num );
    //remove the Vg by its name
    bool removeVg( const QString & vgName );
    //a list of pointers to the Vgs
    QPtrList<Vg> vgList;
    //this list holds the names of the VGs which have corrupted VGDA
    QStringList errorVgNameList;
};
#endif
